# backwards compat - moved to own python package
from ovos_config.locale import setup_locale, set_default_tz, set_default_lang, set_default_lf_lang, load_language, load_languages, get_default_lang, get_default_tz, get_config_tz, get_primary_lang_code
