# backwards compat imports
from ovos_gui.namespace import NamespaceManager
from ovos_gui.extensions import ExtensionsManager
from ovos_gui.service import GUIService, on_started, on_alive, on_ready, on_error, on_stopping

