"""
This module contains back compat imports only
Speech client moved into mycroft.listener module
"""
from ovos_listener.data_structures import CyclicAudioBuffer
from mycroft.deprecated.speech_client import RollingMean
