#!/bin/bash

lupdate settingspage.qml -ts settingspage_de.ts
lupdate settingspage.qml -ts settingspage_es.ts
lupdate settingspage.qml -ts settingspage_fr.ts
lupdate settingspage.qml -ts settingspage_nl.ts
lupdate settingspage.qml -ts settingspage_pt.ts
lupdate settingspage.qml -ts settingspage_it.ts

lupdate homescreen_settings.qml -ts homescreen_settings_de.ts
lupdate homescreen_settings.qml -ts homescreen_settings_es.ts
lupdate homescreen_settings.qml -ts homescreen_settings_fr.ts
lupdate homescreen_settings.qml -ts homescreen_settings_nl.ts
lupdate homescreen_settings.qml -ts homescreen_settings_pt.ts
lupdate homescreen_settings.qml -ts homescreen_settings_it.ts

lupdate developer_settings.qml -ts developer_settings_de.ts
lupdate developer_settings.qml -ts developer_settings_es.ts
lupdate developer_settings.qml -ts developer_settings_fr.ts
lupdate developer_settings.qml -ts developer_settings_nl.ts
lupdate developer_settings.qml -ts developer_settings_pt.ts
lupdate developer_settings.qml -ts developer_settings_it.ts

lupdate display_settings.qml -ts display_settings_de.ts
lupdate display_settings.qml -ts display_settings_es.ts
lupdate display_settings.qml -ts display_settings_fr.ts
lupdate display_settings.qml -ts display_settings_nl.ts
lupdate display_settings.qml -ts display_settings_pt.ts
lupdate display_settings.qml -ts display_settings_it.ts

lupdate customize_settings.qml -ts customize_settings_de.ts
lupdate customize_settings.qml -ts customize_settings_es.ts
lupdate customize_settings.qml -ts customize_settings_fr.ts
lupdate customize_settings.qml -ts customize_settings_nl.ts
lupdate customize_settings.qml -ts customize_settings_pt.ts
lupdate customize_settings.qml -ts customize_settings_it.ts

lupdate customize_theme.qml -ts customize_theme_de.ts
lupdate customize_theme.qml -ts customize_theme_es.ts
lupdate customize_theme.qml -ts customize_theme_fr.ts
lupdate customize_theme.qml -ts customize_theme_nl.ts
lupdate customize_theme.qml -ts customize_theme_pt.ts
lupdate customize_theme.qml -ts customize_theme_it.ts

lupdate configuration_groups_display.qml -ts configuration_groups_display_de.ts
lupdate configuration_groups_display.qml -ts configuration_groups_display_es.ts
lupdate configuration_groups_display.qml -ts configuration_groups_display_fr.ts
lupdate configuration_groups_display.qml -ts configuration_groups_display_nl.ts
lupdate configuration_groups_display.qml -ts configuration_groups_display_pt.ts
lupdate configuration_groups_display.qml -ts configuration_groups_display_it.ts

lupdate configuration_generator_display.qml -ts configuration_generator_display_de.ts
lupdate configuration_generator_display.qml -ts configuration_generator_display_es.ts
lupdate configuration_generator_display.qml -ts configuration_generator_display_fr.ts
lupdate configuration_generator_display.qml -ts configuration_generator_display_nl.ts
lupdate configuration_generator_display.qml -ts configuration_generator_display_pt.ts
lupdate configuration_generator_display.qml -ts configuration_generator_display_it.ts

lupdate delegate/SetNameBox.qml -ts SetNameBox_de.ts
lupdate delegate/SetNameBox.qml -ts SetNameBox_es.ts
lupdate delegate/SetNameBox.qml -ts SetNameBox_fr.ts
lupdate delegate/SetNameBox.qml -ts SetNameBox_nl.ts
lupdate delegate/SetNameBox.qml -ts SetNameBox_pt.ts
lupdate delegate/SetNameBox.qml -ts SetNameBox_it.ts

lupdate configuration_ui/settingListBox.qml -ts settingListBox_de.ts
lupdate configuration_ui/settingListBox.qml -ts settingListBox_es.ts
lupdate configuration_ui/settingListBox.qml -ts settingListBox_fr.ts
lupdate configuration_ui/settingListBox.qml -ts settingListBox_nl.ts
lupdate configuration_ui/settingListBox.qml -ts settingListBox_pt.ts
lupdate configuration_ui/settingListBox.qml -ts settingListBox_it.ts
