# backwards compat imports
from ovos_audio.tts import TTSFactory
from ovos_plugin_manager.tts import load_tts_plugin
from ovos_plugin_manager.templates.tts import TTS, RemoteTTS, \
    RemoteTTSException, RemoteTTSTimeoutException