# backwards compat imports
from ovos_listener.hotword_factory import HotWordFactory
from ovos_plugin_manager.wakewords import load_wake_word_plugin, find_wake_word_plugins
from ovos_plugin_manager.templates.hotwords import HotWordEngine