"""
This module contains back compat imports only
Speech client moved into mycroft.listener module
"""
from ovos_listener.hotword_factory import HotWordFactory
from ovos_plugin_manager.templates.hotwords import HotWordEngine

