# backwards compat imports
from ovos_audio.tts import TTSFactory
from ovos_plugin_manager.templates.tts import TTS
from ovos_audio.audio import AudioService
from ovos_audio.service import PlaybackService, on_ready, on_alive, on_started, on_error, on_stopping